/*
** Module   : WebHandler.java
** Abstract : starts ChuiWebClient and redirect to the embedded server URI.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20131113 Created initial version
** 002 MAG 20131129 Add authentication page. Handle GET and POST requests.
*/

package com.goldencode.p2j.main;

import java.io.*;
import java.util.logging.*;

import javax.servlet.*;
import javax.servlet.http.*;

import org.eclipse.jetty.http.MimeTypes;
import org.eclipse.jetty.server.*;
import org.eclipse.jetty.server.handler.*;

import com.goldencode.p2j.ui.client.chui.driver.web.*;
import com.goldencode.p2j.util.*;
import com.goldencode.util.PlatformHelper;

/**
 * Starts a web client and redirect the request to the embedded server URI.
 * <p>
 * On GET method a web page containing an authentication form is returned to caller.
 * The user should enter the user name and the password used to authenticate on
 * the server platform (Linux / Windows) and press the "Log in" button to submit the form.
 * <p>
 * On POST method using the user submitted credentials a new web client process is spawned
 * on user environment. On web client URI notification the user is redirected to this URI. 
 * If no response from spawned client occur in a specified amount of time the authentication 
 * page and an error message is returned.
 * 
 *  @author mag
 *  
 */
public class WebHandler 
extends AbstractHandler 
{
   /** Logger. */
   private static final Logger LOG = LogHelper.getLogger(WebHandler.class.getName());

   /** The root portion of the target request name (part of the URL). */
   private String targetRoot = "/chui";
   
   /** Error message */
   private String errorMessage = "";
   
   /**
    * Handle HTTP requests.
    * 
    * @param   target
    *          The target of the request - either a URI or a name.
    * @param   base
    *          The base request.
    * @param   request
    *          The object or a wrapper of the request.
    * @param   response
    *          The object or a wrapper of the response.
    */
   public void handle(String target, 
                      Request base, 
                      HttpServletRequest request,
                      HttpServletResponse response) 
   throws IOException, 
          ServletException
   {
      if (!base.isHandled() && targetRoot.equalsIgnoreCase(target))
      {
         // Handle GET requests.
         if ("GET".equalsIgnoreCase(base.getMethod()))
         {
            doGet(target, base, request, response);
         }
         // Handle POST requests.
         else if ("POST".equalsIgnoreCase(base.getMethod()))
         {
            doPost(target, base, request, response);
         }
      }
   }

   /**
    * Handle GET requests. Send a Login dialog page back to user.
    * 
    * @param   target
    *          The target of the request - either a URI or a name.
    * @param   base
    *          The base request.
    * @param   request
    *          The object or a wrapper of the request.
    * @param   response
    *          The object or a wrapper of the response.
    */
   public void doGet(String target, Request base, HttpServletRequest request,
            HttpServletResponse response) throws IOException, ServletException
   {
      
      // produce the page contents
      response.setContentType(MimeTypes.Type.TEXT_HTML.asString());
      response.setStatus(HttpServletResponse.SC_OK);
      
      PrintWriter writer = response.getWriter();
      
      try
      {
         StringBuilder sb = new StringBuilder("/");
         String packageName = ChuiWebPageHandler.class.getPackage().getName();
         sb.append(packageName.replace('.', '/')).append("/chui_web.html").toString();
         
         InputStream is = this.getClass().getResourceAsStream(sb.toString());
         BufferedReader reader = new BufferedReader(new InputStreamReader(is));
         
         String str = null;
         
         while (null != (str = reader.readLine()))
         {
            writer.println(handleReplacements(str));
         }
         
         reader.close();
      }
      catch (IOException ioe)
      {
         LOG.logp(Level.SEVERE,
                  "WebHandler.doGet()",
                  "",
                  LogHelper.generate("IOException!"),
                  ioe);
      }
      finally
      {
         // mark the request as handled
         base.setHandled(true);
      }      
   }
   
   /**
    * Handle POST request.
    * Spawn web client and redirect the request to the embedded server.
    * 
    * @param   target
    *          The target of the request - either a URI or a name.
    * @param   base
    *          The base request.
    * @param   request
    *          The object or a wrapper of the request.
    * @param   response
    *          The object or a wrapper of the response.
    */
   public void doPost(String target, Request base, HttpServletRequest request,
            HttpServletResponse response) throws IOException, ServletException
   {
      // Get form parameters;
      String userName = base.getParameter("usr");
      String password = base.getParameter("psw");
      // Spawn a new chui web client.
      WebClientSpawner webSpawner = new WebClientSpawner();
      String remoteUri = webSpawner.spawn(userName, password);
      
      if (remoteUri != null)
      {
         LOG.logp(Level.INFO, 
                  "WebHandler.handle", 
                  "",
                  LogHelper.generate("Redirect to " + remoteUri));
         base.setHandled(true);
         response.sendRedirect(remoteUri);
      }
      else
      {
         int exitCode = webSpawner.getExitCode();
         StringBuilder sb = new StringBuilder();
         sb.append("<h1 style=\"color:red\">");
         if (PlatformHelper.isUnderWindowsFamily())
         {
            switch (exitCode)
            {
               case -2:
                  sb.append("Invalid user name or password does not match.");
                  break;
               case -3:
                  sb.append("Cannot create user environment.");
                  break;
               case -4:
                  sb.append("Cannot get user profile.");
                  break;
               case -5:
                  sb.append("Cannot set working directory.");
                  break;
               case -6:
                  sb.append("Cannot create process for user ").append(userName);
                  break;
               case -7:
                  sb.append("Cannot load dynamic libaries.");
                  break;
               default:
                  sb.append("Cannot start the embedded server.");
                  break;
            }            
         }
         else
         {
            switch (exitCode)
            {
               case 1:
                  sb.append("Invalid user name.");
                  break;
               case 2:
                  sb.append("Password does not match.");
                  break;
               case 3:
                  sb.append("User root not allowed.");
                  break;
               case 4:
                  sb.append("PAM authentication fails.");
                  break;
               case -1:
                  sb.append("Spawner has no root access permissions.");
                  break;
               default:
                  sb.append("Cannot start the embedded server.");
                  break;
            }
         }
         sb.append("</h1>");
         errorMessage = sb.toString();
         doGet(target, base, request, response);
         errorMessage = "";
      }      
   }
   
   /**
    * Process the given string and make any replacements of parameter
    * values as needed. The replacement placeholder follow the JEE 
    * expression language (EL) format for immediate evaluation. 
    * <p>
    * Example: ${expression} 
    *
    * @param    text
    *           The text line that may need modifications.
    *
    * @return   The new text if a modification occurred or the original text
    *           if no modification was needed.
    */
   protected String handleReplacements(String text)
   {
      if (text.contains("${error}"))
      {
         text = text.replace("${error}", errorMessage);
      }
      
      return text;
   }   
}

